from functools import wraps
from crf_tagger.utils import flatten


def _flattens_y(func):
    @wraps(func)
    def wrapper(y_true, y_pred, *args, **kwargs):
        y_true_flat = flatten(y_true)
        y_pred_flat = flatten(y_pred)
        return func(y_true_flat, y_pred_flat, *args, **kwargs)

    return wrapper


@_flattens_y
def flat_accuracy_score(y_true, y_pred):
    """
    Return accuracy score for sequence items.
    """
    from sklearn import metrics

    return metrics.accuracy_score(y_true, y_pred)


@_flattens_y
def flat_precision_score(y_true, y_pred, **kwargs):
    """
    Return precision score for sequence items.
    """
    from sklearn import metrics

    return metrics.precision_score(y_true, y_pred, **kwargs)


@_flattens_y
def flat_recall_score(y_true, y_pred, **kwargs):
    """
    Return recall score for sequence items.
    """
    from sklearn import metrics

    return metrics.recall_score(y_true, y_pred, **kwargs)


@_flattens_y
def flat_f1_score(y_true, y_pred, **kwargs):
    """
    Return F1 score for sequence items.
    """
    from sklearn import metrics

    return metrics.f1_score(y_true, y_pred, **kwargs)


@_flattens_y
def flat_fbeta_score(y_true, y_pred, beta, **kwargs):
    """
    Return F-beta score for sequence items.
    """
    from sklearn import metrics

    return metrics.fbeta_score(y_true, y_pred, beta, **kwargs)


@_flattens_y
def flat_classification_report(y_true, y_pred, labels=None, **kwargs):
    """
    Return classification report for sequence items.
    """
    from sklearn import metrics

    return metrics.classification_report(y_true, y_pred, labels, **kwargs)


def sequence_accuracy_score(y_true, y_pred):
    """
    Return sequence accuracy score. Match is counted only when two sequences
    are equal.
    """
    total = len(y_true)
    if not total:
        return 0

    matches = sum(
        1 for yseq_true, yseq_pred in zip(y_true, y_pred) if yseq_true == yseq_pred
    )

    return matches / total



"""
Scorer functions to use with scikit-learn ``cross_val_score``,
``GridSearchCV``, ``RandomizedSearchCV`` and other similar classes.
"""
from sklearn.metrics import make_scorer

from crf_tagger import metrics


flat_accuracy = make_scorer(metrics.flat_accuracy_score)
sequence_accuracy = make_scorer(metrics.sequence_accuracy_score)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['crf_tagger']

package_data = \
{'': ['*']}

install_requires = \
['python-crfsuite==0.9.7', 'scikit-learn', 'tabulate', 'tqdm']

setup_kwargs = {
    'name': 'crf-tagger',
    'version': '0.3.0',
    'description': 'CRF tagger',
    'long_description': None,
    'author': 'severinsimmler',
    'author_email': 's.simmler@snapaddy.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import pycrfsuite
from tabulate import tabulate


class LinePerIterationTrainer(pycrfsuite.Trainer):
    """
    This pycrfsuite.Trainer prints information about each iteration
    on a single line.
    """

    def on_iteration(self, log, info):
        parts = [
            "Iter {num:<3} ",
            "time={time:<5.2f} ",
            "loss={loss:<8.2f} ",
        ]

        if "active_features" in info:
            parts += ["active={active_features:<5} "]

        if "avg_precision" in info:
            parts += [
                "precision={avg_precision:0.3f}  ",
                "recall={avg_recall:0.3f}  ",
                "F1={avg_f1:0.3f}  ",
                "Acc(item/seq)={item_accuracy_float:0.3f} {instance_accuracy_float:0.3f}  ",
            ]

        if "feature_norm" in info:
            parts += ["feature_norm={feature_norm:<8.2f}"]

        line = "".join(parts)
        print(line.format(**info).strip())

    def on_optimization_end(self, log):
        last_iter = self.logparser.last_iteration
        if last_iter.get("scores", None):
            data = [
                [entity, score.precision, score.recall, score.f1 or 0, score.ref]
                for entity, score in sorted(last_iter["scores"].items())
            ]
            table = tabulate(
                data,
                headers=["Label", "Precision", "Recall", "F1", "Support"],
                floatfmt="0.3f",
            )
            size = len(table.splitlines()[0])
            print("=" * size)
            print(table)
            print("-" * size)
        super(LinePerIterationTrainer, self).on_optimization_end(log)
